@@ -81,6 +81,13 @@ from .optical.spot45_product import (
     Spot45Product,
 )
 
+# Planet
+__all__ += [
+    "PlanetMaskType",
+]
+
+from .optical.planet_product import PlanetMaskType
+
 # PlanetScope
 __all__ += [
     "PlaProduct",
@@ -97,6 +104,14 @@ __all__ += [
 ]
 from .optical.sky_product import SkyProductType, SkyProduct, SkyInstrument
 
+# RapidEye
+__all__ += [
+    "ReProductType",
+    "ReProduct",
+    "ReInstrument",
+]
+from .optical.re_product import ReProductType, ReProduct
+
 # Landsat
 __all__ += [
     "LandsatProduct",
