@@ -17,27 +17,30 @@
 """
 PlanetScope products.
 See
-`Earth Online <https://assets.planet.com/docs/Planet_Combined_Imagery_Product_Specs_letter_screen.pdf>`_
+`Product specs <https://assets.planet.com/docs/Planet_Combined_Imagery_Product_Specs_letter_screen.pdf>`_
 and `Planet documentation <https://developers.planet.com/docs/data/planetscope/>`_
 for more information.
 """
 import logging
+from collections import defaultdict
 from datetime import datetime
 from enum import unique
 from pathlib import Path
 from typing import Union
 
+import numpy as np
+import rasterio
 import xarray as xr
 from cloudpathlib import CloudPath
 from lxml import etree
+from sertit import files, rasters, xml
 from sertit.misc import ListEnum
+from sertit.vectors import WGS84
 
-from eoreader import cache
 from eoreader.bands import BandNames, SpectralBand
 from eoreader.bands import spectral_bands as spb
 from eoreader.exceptions import InvalidProductError
 from eoreader.products.optical.planet_product import PlanetProduct
-from eoreader.products.product import OrbitDirection
 from eoreader.stac import GSD, ID, NAME, WV_MAX, WV_MIN
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME
 
@@ -47,7 +50,7 @@ LOGGER = logging.getLogger(EOREADER_NAME)
 @unique
 class PlaInstrument(ListEnum):
     """PlanetScope instrument
-    See `here <https://developers.planet.com/docs/apis/data/sensors/>`__
+    See `Planet documentation <https://developers.planet.com/docs/apis/data/sensors/>`__
     for more information.
     """
 
@@ -136,7 +139,7 @@ class PlaProductType(ListEnum):
     basic_analytic_8b_xml   Unorthorectified radiometrically-calibrated analytic image metadata
     basic_analytic_4b_rpc   RPC for unorthorectified analytic image stored as 12-bit digital numbers.
     basic_analytic_4b_xml   Unorthorectified radiometrically-calibrated analytic image metadata.
-    basic_udm2              Unorthorectified usable data mask (Cloud 2.0) Read more about this new asset here.
+    basic_udm2              Unorthorectified usable data mask (Cloud 2.0)
     ortho_udm2              Usable data mask (Cloud 2.0)
     ortho_visual            Visual image with color-correction
     """
@@ -188,10 +191,6 @@ class PlaProduct(PlanetProduct):
         """
         self._has_cloud_cover = True
 
-        # Ortho Tiles
-        if self.product_type == PlaProductType.L3A:
-            self.tile_name = self.split_name[1]
-
         # Post init done by the super class
         super()._post_init(**kwargs)
 
@@ -364,7 +363,7 @@ class PlaProduct(PlanetProduct):
         root, nsmap = self.read_mtd()
 
         # Manage bands of the product
-        nof_bands = int(root.findtext(f".//{nsmap['ps']}numBands"))
+        nof_bands = int(root.findtext(f".//{nsmap[self._nsmap_key]}numBands"))
 
         # Set the band map
         self.bands.map_bands(
@@ -389,11 +388,6 @@ class PlaProduct(PlanetProduct):
             raise NotImplementedError(
                 f"Basic Scene Product are not managed for Planet products {self.path}"
             )
-        elif self.product_type == PlaProductType.L3A:
-            LOGGER.warning(
-                f"Ortho Tile Product are not well tested for Planet products {self.path}."
-                f"Use it at your own risk !"
-            )
 
     def get_datetime(self, as_datetime: bool = False) -> Union[str, datetime]:
         """
@@ -421,42 +415,44 @@ class PlaProduct(PlanetProduct):
             datetime_str = root.findtext(f".//{nsmap['eop']}acquisitionDate")
             if not datetime_str:
                 raise InvalidProductError(
-                    "Cannot find EARLIESTACQTIME in the metadata file."
+                    "Cannot find acquisitionDate in the metadata file."
                 )
 
             # Convert to datetime
-            datetime_str = datetime.strptime(
-                datetime_str.split("+")[0], "%Y-%m-%dT%H:%M:%S"
-            )
-
-            if not as_datetime:
-                datetime_str = datetime_str.strftime(DATETIME_FMT)
+            datetime_str = datetime_str.split("+")[0]
+            try:
+                datetime_str = datetime.strptime(datetime_str, "%Y-%m-%dT%H:%M:%S")
+            except ValueError:
+                datetime_str = datetime.strptime(datetime_str, "%Y-%m-%dT%H:%M:%S.%f")
 
         else:
             datetime_str = self.datetime
-            if not as_datetime:
-                datetime_str = datetime_str.strftime(DATETIME_FMT)
+
+        if not as_datetime:
+            datetime_str = datetime_str.strftime(DATETIME_FMT)
 
         return datetime_str
 
-    def _get_name_constellation_specific(self) -> str:
+    def _get_stack_path(self, as_list: bool = False) -> Union[str, list]:
         """
-        Set product real name from metadata
+        Get Planet stack path(s)
+
+        Args:
+            as_list (bool): Get stack path as a list (useful if several subdatasets are present)
 
         Returns:
-            str: True name of the product (from metadata)
+            Union[str, list]: Stack path(s)
         """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
-
-        # Open identifier
-        name = root.findtext(f".//{nsmap['eop']}identifier")
-        if not name:
-            raise InvalidProductError(
-                f"{nsmap['eop']}identifier not found in metadata!"
+        if self._merged:
+            stack_path, _ = self._get_out_path(f"{self.condensed_name}_analytic.vrt")
+            if as_list:
+                stack_path = [stack_path]
+        else:
+            stack_path = self._get_path(
+                "Analytic", "tif", invalid_lookahead="udm", as_list=as_list
             )
 
-        return name
+        return stack_path
 
     def get_band_paths(
         self, band_list: list, resolution: float = None, **kwargs
@@ -487,10 +483,9 @@ class PlaProduct(PlanetProduct):
             dict: Dictionary containing the path of each queried band
         """
         band_paths = {}
+        path = self._get_stack_path(as_list=False)
         for band in band_list:
-            band_paths[band] = self._get_path(
-                "AnalyticMS", "tif", invalid_lookahead="_DN_"
-            )
+            band_paths[band] = path
 
         return band_paths
 
@@ -513,187 +508,236 @@ class PlaProduct(PlanetProduct):
         Returns:
             xr.DataArray: Band in reflectance
         """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
+        if self._merged:
+            return band_arr
+        else:
+            # Get MTD XML file
+            root, nsmap = self.read_mtd()
 
-        # Open identifier
-        refl_coef = None
-        for band_mtd in root.iterfind(f".//{nsmap['ps']}bandSpecificMetadata"):
-            if (
-                int(band_mtd.findtext(f".//{nsmap['ps']}bandNumber"))
-                == self.bands[band].id
+            # Open identifier
+            refl_coef = None
+
+            for band_mtd in root.iterfind(
+                f".//{nsmap[self._nsmap_key]}bandSpecificMetadata"
             ):
-                refl_coef = float(
-                    band_mtd.findtext(f".//{nsmap['ps']}reflectanceCoefficient")
+                if (
+                    int(band_mtd.findtext(f".//{nsmap[self._nsmap_key]}bandNumber"))
+                    == self.bands[band].id
+                ):
+                    refl_coef = float(
+                        band_mtd.findtext(
+                            f".//{nsmap[self._nsmap_key]}reflectanceCoefficient"
+                        )
+                    )
+                    break
+
+            if refl_coef is None:
+                raise InvalidProductError(
+                    "Couldn't find any reflectanceCoefficient in the product metadata!"
                 )
-                break
 
-        if refl_coef is None:
-            raise InvalidProductError(
-                "Couldn't find any reflectanceCoefficient in the product metadata!"
+            # To reflectance
+            return band_arr * refl_coef
+
+    def _merge_subdatasets_mtd(self):
+        """
+        Merge subdataset, when several Planet products avec been ordered together
+        Will create a reflectance (if possible) VRT, a UDM/UDM2 VRT and a merged metadata XML file.
+        """
+
+        def update_corner_dict(key, lon, lat):
+            try:
+                lon = lon.iat[0]
+            except Exception:
+                pass
+            try:
+                lat = lat.iat[0]
+            except Exception:
+                pass
+            xml.update_txt(
+                mtd,
+                f"{nsmap[self._nsmap_key]}{key}/{nsmap[self._nsmap_key]}longitude",
+                lon,
+            )
+            xml.update_txt(
+                mtd,
+                f"{nsmap[self._nsmap_key]}{key}/{nsmap[self._nsmap_key]}latitude",
+                lat,
             )
 
-        # To reflectance
-        return band_arr * refl_coef
+        # Merge datasets
+        analytic_vrt_path, analytic_vrt_exists = self._merge_subdatasets()
 
-    @cache
-    def get_mean_sun_angles(self) -> (float, float):
-        """
-        Get Mean Sun angles (Azimuth and Zenith angles)
+        # Check if mtd needs an update
+        mtd_file, mtd_exists = self._get_out_path(f"{self.condensed_name}_metadata.xml")
 
-        .. code-block:: python
+        # -- Update VRT
+        scales = defaultdict(dict)
+        cloud_cover = []
+        udp = []
+        if not mtd_exists or not analytic_vrt_exists:
+            # Get all scales, cloud cloudCoverPercentage, unusableDataPercentage
+            for mtd_f in self._get_path("metadata", "xml", as_list=True):
+                mtd_filename = files.get_filename(mtd_f)
+                subprod_name = mtd_filename.split("_Analytic")[0]
+                mtd, nsmap = self._read_mtd_xml(
+                    f"{subprod_name}*metadata*xml", f"{subprod_name}.*metadata.*xml"
+                )
 
-            >>> from eoreader.reader import Reader
-            >>> path = r"SENTINEL2A_20190625-105728-756_L2A_T31UEQ_C_V2-2"
-            >>> prod = Reader().open(path)
-            >>> prod.get_mean_sun_angles()
-            (154.554755774838, 27.5941391571236)
+                # reflectanceCoefficient
+                for band_mtd in mtd.iterfind(
+                    f".//{nsmap[self._nsmap_key]}bandSpecificMetadata"
+                ):
+                    band_nb = band_mtd.findtext(f"{nsmap[self._nsmap_key]}bandNumber")
+                    refl_coef = band_mtd.findtext(
+                        f"{nsmap[self._nsmap_key]}reflectanceCoefficient"
+                    )
+                    scales[subprod_name][band_nb] = refl_coef
 
-        Returns:
-            (float, float): Mean Azimuth and Zenith angle
-        """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
+                # cloudCoverPercentage
+                cloud_cover.append(
+                    float(mtd.findtext(f".//{nsmap['opt']}cloudCoverPercentage"))
+                )
 
-        # Open zenith and azimuth angle
-        try:
-            elev_angle = float(
-                root.findtext(f".//{nsmap['opt']}illuminationElevationAngle")
+                # unusableDataPercentage
+                udp.append(
+                    float(
+                        mtd.findtext(
+                            f".//{nsmap[self._nsmap_key]}unusableDataPercentage"
+                        )
+                    )
+                )
+
+        if not analytic_vrt_exists:
+            LOGGER.debug("Update raster VRT")
+            vrt = etree.parse(analytic_vrt_path).getroot()
+
+            # Remove stats and histograms
+            xml.remove(vrt, "Metadata")
+            xml.remove(vrt, "Histograms")
+
+            # Convert to Float32
+            xml.update_attrib(
+                vrt, "VRTRasterBand[@dataType='UInt16']", "dataType", "Float32"
+            )  # datatype with d!
+            xml.update_attrib(
+                vrt, "SourceProperties[@DataType='UInt16']", "DataType", "Float32"
+            )  # datatype with D!
+
+            # Scale the VRT
+            for el in vrt.iterfind(".//ComplexSource"):
+                band_name = files.get_filename(el.findtext("SourceFilename")).split(
+                    "_Analytic"
+                )[0]
+                band_number = el.findtext("SourceBand")
+
+                # Set scaleRatio in VRT
+                xml.add(el, "ScaleRatio", scales[band_name][band_number])
+
+            # Write VRT on disk
+            xml.write(vrt, analytic_vrt_path)
+
+        # -- Update MTD
+        if not mtd_exists:
+            LOGGER.debug("Merge metadata")
+            mtd, nsmap = self.read_mtd()
+
+            # Remove all reflectance scaling
+            xml.remove(mtd, f"{nsmap[self._nsmap_key]}reflectanceCoefficient")
+
+            # Get new size from VRT
+            with rasterio.open(str(analytic_vrt_path)) as ds:
+                xml.update_txt(mtd, f"{nsmap[self._nsmap_key]}numRows", ds.height)
+                xml.update_txt(mtd, f"{nsmap[self._nsmap_key]}numColumns", ds.width)
+
+            # Get new extent from VRT
+            extent = rasters.get_extent(analytic_vrt_path)
+            extent_wgs84 = extent.to_crs(WGS84)
+
+            # Compute centroid and reproject to WGS84 after
+            pos = extent.centroid.to_crs(WGS84).iat[0]
+            xml.update_txt(mtd, f"{nsmap['gml']}pos", f"{pos.x} {pos.y}")
+
+            # Get extent coordinates (should be footprint but too long to compute IMHO)
+            coordinates_str = " ".join(
+                f"{coord[0]},{coord[1]}"
+                for coord in extent_wgs84.boundary.iat[0].coords
             )
-            azimuth_angle = float(
-                root.findtext(f".//{nsmap['opt']}illuminationAzimuthAngle")
+            xml.update_txt(mtd, f"{nsmap['gml']}coordinates", coordinates_str)
+
+            # Get corners
+            bounds_wgs84 = extent_wgs84.bounds
+            update_corner_dict("topLeft", bounds_wgs84.maxx, bounds_wgs84.miny)
+            update_corner_dict("topRight", bounds_wgs84.maxx, bounds_wgs84.maxy)
+            update_corner_dict("bottomRight", bounds_wgs84.minx, bounds_wgs84.maxy)
+            update_corner_dict("bottomLeft", bounds_wgs84.minx, bounds_wgs84.miny)
+
+            # Manage cloudCoverPercentage, unusableDataPercentage
+            xml.update_txt(
+                mtd, f"{nsmap['opt']}cloudCoverPercentage", np.mean(cloud_cover)
             )
-        except TypeError:
-            raise InvalidProductError("Azimuth or Zenith angles not found in metadata!")
-
-        # From elevation to zenith
-        zenith_angle = 90.0 - elev_angle
-
-        return azimuth_angle, zenith_angle
-
-    @cache
-    def get_mean_viewing_angles(self) -> (float, float, float):
-        """
-        Get Mean Viewing angles (azimuth, off-nadir and incidence angles)
-
-        .. code-block:: python
-
-            >>> from eoreader.reader import Reader
-            >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
-            >>> prod = Reader().open(path)
-            >>> prod.get_mean_viewing_angles()
-
-        Returns:
-            (float, float, float): Mean azimuth, off-nadir and incidence angles
-        """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
-
-        # Open zenith and azimuth angle
-        try:
-            az = float(root.findtext(f".//{nsmap['ps']}azimuthAngle"))
-            off_nadir = float(root.findtext(f".//{nsmap['ps']}spaceCraftViewAngle"))
-            incidence_angle = float(root.findtext(f".//{nsmap['eop']}incidenceAngle"))
-        except TypeError:
-            raise InvalidProductError(
-                "azimuthAngle, spaceCraftViewAngle or incidenceAngle not found in metadata!"
+            xml.update_txt(
+                mtd, f"{nsmap[self._nsmap_key]}unusableDataPercentage", np.mean(udp)
             )
 
-        return az, off_nadir, incidence_angle
+            if self.product_type == PlaProductType.L3A:
+                # -- PSOrthoTile
 
-    @cache
-    def _read_mtd(self) -> (etree._Element, dict):
-        """
-        Read metadata and outputs the metadata XML root and its namespaces as a dict
+                # identifier: keep the one opened
 
-        .. code-block:: python
+                # Remove tileId
+                xml.remove(mtd, f"{nsmap[self._nsmap_key]}tileId")
 
-            >>> from eoreader.reader import Reader
-            >>> path = r"20210406_015904_37_2407.zip"
-            >>> prod = Reader().open(path)
-            >>> prod.read_mtd()
-            (<Element {http://schemas.planet.com/ps/v1/planet_product_metadata_geocorrected_level}
-            EarthObservation at 0x1a2621f03c8>,
-            {
-                'opt': '{http://earth.esa.int/opt}',
-                'gml': '{http://www.opengis.net/gml}',
-                'eop': '{http://earth.esa.int/eop}',
-                'ps': '{http://schemas.planet.com/ps/v1/planet_product_metadata_geocorrected_level}'
-            })
+                # Round incidenceAngle
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap['eop']}incidenceAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
 
-        Returns:
-            (etree._Element, dict): Metadata XML root and its namespaces as a dict
-        """
-        mtd_from_path = "metadata*.xml"
-        mtd_archived = r"metadata.*\.xml"
+            elif self.product_type == PlaProductType.L3B:
+                # -- PSOrthoScene
+                # identifier: replace satellite ID by XX: 20210902_093940_06_245d_3B_AnalyticMS_8b -> 20210902_093940_XX_245d_3B_AnalyticMS_8b
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap['eop']}identifier",
+                    lambda x: "_".join(
+                        "XX" if i == 2 else z for i, z in enumerate(x.split("_"))
+                    ),
+                )
 
-        return self._read_mtd_xml(mtd_from_path, mtd_archived)
+                # Remove filename
+                xml.remove(mtd, f"{nsmap['eop']}fileName")
 
-    @cache
-    def get_cloud_cover(self) -> float:
-        """
-        Get cloud cover as given in the metadata
+                # Round incidenceAngle, illuminationAzimuthAngle, illuminationElevationAngle, azimuthAngle, spaceCraftViewAngle
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap['eop']}incidenceAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap['opt']}illuminationAzimuthAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap['opt']}illuminationElevationAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap[self._nsmap_key]}azimuthAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
+                xml.update_txt_fct(
+                    mtd,
+                    f"{nsmap[self._nsmap_key]}spaceCraftViewAngle",
+                    lambda x: np.round(float(x), decimals=1),
+                )
 
-        .. code-block:: python
+            else:
+                raise NotImplementedError
 
-            >>> from eoreader.reader import Reader
-            >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
-            >>> prod = Reader().open(path)
-            >>> prod.get_cloud_cover()
-            55.5
-
-        Returns:
-            float: Cloud cover as given in the metadata
-        """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
-
-        # Get the cloud cover
-        try:
-            cc = float(root.findtext(f".//{nsmap['opt']}cloudCoverPercentage"))
-
-        except TypeError:
-            raise InvalidProductError("opt:cloudCoverPercentage not found in metadata!")
-
-        return cc
-
-    @cache
-    def get_orbit_direction(self) -> OrbitDirection:
-        """
-        Get cloud cover as given in the metadata
-
-        .. code-block:: python
-
-            >>> from eoreader.reader import Reader
-            >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
-            >>> prod = Reader().open(path)
-            >>> prod.get_orbit_direction().value
-            "DESCENDING"
-
-        Returns:
-            OrbitDirection: Orbit direction (ASCENDING/DESCENDING)
-        """
-        # Get MTD XML file
-        root, nsmap = self.read_mtd()
-
-        # Get the orbit direction
-        try:
-            od = OrbitDirection.from_value(
-                root.findtext(f".//{nsmap['eop']}orbitDirection")
-            )
-
-        except TypeError:
-            raise InvalidProductError("eop:orbitDirection not found in metadata!")
-
-        return od
-
-    def _get_condensed_name(self) -> str:
-        """
-        Get Planet products condensed name ({date}_{constellation}_{product_type}).
-
-        Returns:
-            str: Condensed name
-        """
-        return (
-            f"{self.get_datetime()}_{self.constellation.name}_{self.product_type.name}"
-        )
+            # Write XML on disk
+            xml.write(mtd, mtd_file)
