@@ -518,6 +518,12 @@ class S3OlciProduct(S3Product):
                 filename, subdataset, dtype=kwargs.get("dtype", np.float32)
             )
 
+            # Convert radiance to reflectances if needed
+            # Convert first pixel by pixel before reprojection !
+            if to_reflectance:
+                LOGGER.debug(f"Converting {band_str} to reflectance")
+                band_arr = self._rad_2_refl(band_arr, band)
+
             # Geocode
             LOGGER.debug(f"Geocoding {band_str}")
             pp_arr = self._geocode(band_arr, resolution=resolution)
@@ -579,21 +585,19 @@ class S3OlciProduct(S3Product):
         Returns:
             dict: Dictionary containing {band: path}
         """
-        rad_2_refl_path = self._get_band_folder() / f"rad_2_refl_{band.name}.npy"
-
-        if not rad_2_refl_path.is_file():
-            rad_2_refl_path = (
-                self._get_band_folder(writable=True) / f"rad_2_refl_{band.name}.npy"
-            )
-
+        rad_2_refl_path, rad_2_refl_exists = self._get_out_path(
+            f"rad_2_refl_{band.name}.npy"
+        )
+        if not rad_2_refl_exists:
             # Open SZA array (resampled to band_arr size)
-            sza_path = self._get_band_folder() / "sza.tif"
-            if not sza_path.is_file():
-                sza_path = self._get_band_folder(writable=True) / "sza.tif"
+            sza_path, sza_exists = self._get_out_path("sza.tif")
+
+            # May have been created before (don't recreate it)
+            if not sza_exists and not sza_path.is_file():
                 sza_nc = self._read_nc(self._geom_file, self._sza_name)
                 utils.write(sza_nc, sza_path)
 
-            with rasterio.open(sza_path) as ds_sza:
+            with rasterio.open(str(sza_path)) as ds_sza:
                 # Values can be easily interpolated at pixels from Tie Points by linear interpolation using the
                 # image column coordinate.
                 sza, _ = rasters_rio.read(
@@ -611,11 +615,11 @@ class S3OlciProduct(S3Product):
             rad_2_refl_coeff = (np.pi / e0 / np.cos(sza_rad)).astype(np.float32)
 
             # Write on disk
-            np.save(rad_2_refl_path, rad_2_refl_coeff)
+            np.save(str(rad_2_refl_path), rad_2_refl_coeff)
 
         else:
             # Open rad_2_refl_coeff (resampled to band_arr size)
-            rad_2_refl_coeff = np.load(rad_2_refl_path)
+            rad_2_refl_coeff = np.load(str(rad_2_refl_path))
 
         return band_arr * rad_2_refl_coeff
 
@@ -757,7 +761,7 @@ class S3OlciProduct(S3Product):
 
     def _has_cloud_band(self, band: BandNames) -> bool:
         """
-        Does this products has the specified cloud band ?
+        Does this product has the specified cloud band ?
         -> OLCI does not provide any cloud mask
         """
         return False
