@@ -37,11 +37,10 @@ import numpy as np
 import xarray as xr
 from cloudpathlib import CloudPath
 from lxml import etree
-from lxml.builder import E
 from rasterio import crs as riocrs
 from rasterio.enums import Resampling
 from rasterio.errors import NotGeoreferencedWarning
-from sertit import files, vectors
+from sertit import files, vectors, xml
 from sertit.misc import ListEnum
 from shapely.geometry import Polygon, box
 
@@ -50,6 +49,7 @@ from eoreader.bands import BandNames
 from eoreader.bands import SpectralBandNames as spb
 from eoreader.exceptions import InvalidProductError
 from eoreader.products import OpticalProduct
+from eoreader.products.optical.optical_product import RawUnits
 from eoreader.reader import Constellation
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME, simplify
 
@@ -141,6 +141,7 @@ class S3Product(OpticalProduct):
         self.needs_extraction = False
         self._use_filename = True
         self.is_ortho = False
+        self._raw_units = RawUnits.RAD
 
         # Post init done by the super class
         super()._pre_init(**kwargs)
@@ -675,18 +676,7 @@ class S3Product(OpticalProduct):
             "track_offset",
         ]
 
-        # Create XML attributes
-        global_attr = []
-        for attr in global_attr_names:
-            if hasattr(netcdf_ds, attr):
-                global_attr.append(E(attr, str(getattr(netcdf_ds, attr))))
-
-        mtd = E.s3_global_attributes(*global_attr)
-        mtd_el = etree.fromstring(
-            etree.tostring(
-                mtd, pretty_print=True, xml_declaration=True, encoding="UTF-8"
-            )
-        )
+        mtd_el = xml.convert_to_xml(netcdf_ds, attributes=global_attr_names)
 
         # Close dataset
         netcdf_ds.close()
