@@ -41,6 +41,7 @@ from eoreader.bands import BandNames, SpectralBand
 from eoreader.bands import spectral_bands as spb
 from eoreader.exceptions import InvalidProductError
 from eoreader.products import VhrProduct
+from eoreader.products.optical.optical_product import RawUnits
 from eoreader.reader import Constellation
 from eoreader.stac import GSD, ID, NAME, WV_MAX, WV_MIN
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME, simplify
@@ -211,6 +212,16 @@ class Spot45Product(VhrProduct):
         self._use_filename = True
         self._proj_prod_type = [Spot45ProductType.L0]
 
+        # Raw units
+        rad_proc = root.findtext(".//RADIOMETRIC_PROCESSING").upper()
+
+        if rad_proc == "REFLECTANCE":
+            self._raw_units = RawUnits.REFL
+        elif rad_proc in "BASIC":
+            self._raw_units = RawUnits.DN
+        else:
+            self._raw_units = RawUnits.NONE
+
         # Post init done by the super class
         super()._pre_init(**kwargs)
 
@@ -731,16 +742,12 @@ class Spot45Product(VhrProduct):
         Returns:
             xr.DataArray: Band in reflectance
         """
-        # Get MTD XML file
-        root, _ = self.read_mtd()
-        rad_proc = root.findtext(".//RADIOMETRIC_PROCESSING").upper()
-
-        if rad_proc == "REFLECTANCE":
+        if self._raw_units == RawUnits.REFL:
             # Compute the correct radiometry of the band
             original_dtype = band_arr.encoding.get("dtype", band_arr.dtype)
             if original_dtype == "uint16":
                 band_arr /= 10000.0
-        elif rad_proc in "BASIC":
+        elif self._raw_units == RawUnits.DN:
             # Convert DN into radiance
             band_arr = self._dn_to_toa_rad(band_arr, band)
 
@@ -776,7 +783,7 @@ class Spot45Product(VhrProduct):
 
     def _has_cloud_band(self, band: BandNames) -> bool:
         """
-        Does this products has the specified cloud band ?
+        Does this product has the specified cloud band ?
         """
         return False
 
@@ -895,7 +902,7 @@ class Spot45Product(VhrProduct):
         try:
             if self.is_archived:
                 quicklook_path = files.get_archived_rio_path(
-                    self.path, file_regex=".*PREVIEW\.JPG"
+                    self.path, file_regex=r".*PREVIEW\.JPG"
                 )
             else:
                 quicklook_path = str(next(self.path.glob("*PREVIEW.JPG")))
