@@ -19,31 +19,32 @@ COSMO-SkyMed 2nd Generation products.
 More info `here <https://egeos.my.salesforce.com/sfc/p/#1r000000qoOc/a/69000000JXxZ/WEEbowzi5cmY8vLqyfAAMKZ064iN1eWw_qZAgUkTtXI>`_.
 """
 import logging
-import warnings
 from datetime import datetime
 from enum import unique
+from io import BytesIO
 from pathlib import Path
 from typing import Union
 
 import geopandas as gpd
+import h5netcdf
 import numpy as np
 import rasterio
+import xarray as xr
 from cloudpathlib import AnyPath, CloudPath
 from lxml import etree
-from sertit import files, strings, vectors
+from lxml.builder import E
+from sertit import files, rasters, strings, vectors
 from sertit.misc import ListEnum
 from shapely.geometry import Polygon, box
 
-from eoreader import cache, cached_property
+from eoreader import cache
 from eoreader.exceptions import InvalidProductError
 from eoreader.products import SarProduct, SarProductType
+from eoreader.products.product import OrbitDirection
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME
 
 LOGGER = logging.getLogger(EOREADER_NAME)
 
-# Disable georef warnings here as the SAR products are not georeferenced
-warnings.filterwarnings("ignore", category=rasterio.errors.NotGeoreferencedWarning)
-
 
 @unique
 class CosmoProductType(ListEnum):
@@ -72,27 +73,6 @@ class CosmoProductType(ListEnum):
     """Level 1D, Geocoded Terrain Corrected"""
 
 
-@unique
-class CosmoPolarization(ListEnum):
-    """
-    COSMO-SkyMed (both generations) polarizations used during the acquisition.
-    More info
-    `here <https://egeos.my.salesforce.com/sfc/p/#1r000000qoOc/a/69000000JXxZ/WEEbowzi5cmY8vLqyfAAMKZ064iN1eWw_qZAgUkTtXI>`_.
-    """
-
-    HH = "HH"
-    """Horizontal Tx/Horizontal Rx for Himage, ScanSAR and Spotlight modes"""
-
-    VV = "VV"
-    """Vertical Tx/Vertical Rx for Himage, ScanSAR and Spotlight modes"""
-
-    HV = "HV"
-    """Horizontal Tx/Vertical Rx for Himage, ScanSAR"""
-
-    VH = "VH"
-    """Vertical Tx/Horizontal Rx for Himage, ScanSAR"""
-
-
 class CosmoProduct(SarProduct):
     """
     Class for COSMO-SkyMed (both generations) Products
@@ -106,6 +86,7 @@ class CosmoProduct(SarProduct):
         archive_path: Union[str, CloudPath, Path] = None,
         output_path: Union[str, CloudPath, Path] = None,
         remove_tmp: bool = False,
+        **kwargs,
     ) -> None:
         try:
             product_path = AnyPath(product_path)
@@ -114,12 +95,11 @@ class CosmoProduct(SarProduct):
             raise InvalidProductError(
                 f"Image file (*.h5) not found in {product_path}"
             ) from ex
-        self._real_name = files.get_filename(self._img_path)
 
         # Initialization from the super class
-        super().__init__(product_path, archive_path, output_path, remove_tmp)
+        super().__init__(product_path, archive_path, output_path, remove_tmp, **kwargs)
 
-    def _pre_init(self) -> None:
+    def _pre_init(self, **kwargs) -> None:
         """
         Function used to pre_init the products
         (setting needs_extraction and so on)
@@ -127,24 +107,24 @@ class CosmoProduct(SarProduct):
         # Private attributes
         self._raw_band_regex = "*_{}_*.h5"
         self._band_folder = self.path
-        self._snap_path = self._img_path.name
+        self.snap_filename = self._img_path.name
 
         # SNAP cannot process its archive
         self.needs_extraction = True
 
         # Post init done by the super class
-        super()._pre_init()
+        super()._pre_init(**kwargs)
 
-    def _post_init(self) -> None:
+    def _post_init(self, **kwargs) -> None:
         """
         Function used to post_init the products
         (setting product-type, band names and so on)
         """
 
         # Post init done by the super class
-        super()._post_init()
+        super()._post_init(**kwargs)
 
-    @cached_property
+    @cache
     def wgs84_extent(self) -> gpd.GeoDataFrame:
         """
         Get the WGS84 extent of the file before any reprojection.
@@ -155,7 +135,7 @@ class CosmoProduct(SarProduct):
             >>> from eoreader.reader import Reader
             >>> path = r"1011117-766193"
             >>> prod = Reader().open(path)
-            >>> prod.wgs84_extent
+            >>> prod.wgs84_extent()
                                                         geometry
             0  POLYGON ((108.09797 15.61011, 108.48224 15.678...
 
@@ -310,7 +290,7 @@ class CosmoProduct(SarProduct):
 
         return date
 
-    def _get_name(self) -> str:
+    def _get_name_constellation_specific(self) -> str:
         """
         Set product real name from metadata
 
@@ -343,6 +323,158 @@ class CosmoProduct(SarProduct):
         Returns:
             (etree._Element, dict): Metadata XML root and its namespaces
         """
-        mtd_from_path = "DFDN_*.h5.xml"
+        try:
+            mtd_from_path = "DFDN_*.h5.xml"
 
-        return self._read_mtd_xml(mtd_from_path)
+            return self._read_mtd_xml(mtd_from_path)
+        except InvalidProductError:
+            try:
+                field_map = {
+                    # ProductInfo
+                    "ProductName": "Product Filename",
+                    # "ProductId": ,
+                    "MissionId": "Mission ID",
+                    # "UniqueIdentifier": ,
+                    "ProductGenerationDate": "Product Generation UTC",
+                    # "UserRequestId": ,
+                    # "ServiceRequestName": ,
+                    # ProductDefinitionData
+                    "ProductType": "Product Type",
+                    "SceneSensingStartUTC": "Scene Sensing Start UTC",
+                    "SceneSensingStopUTC": "Scene Sensing Stop UTC",
+                    # "GeoCoordTopRightEN": ,
+                    "GeoCoordSceneCentre": "Scene Centre Geodetic Coordinates",
+                    "SatelliteId": "Satellite ID",
+                    "AcquisitionMode": "Acquisition Mode",
+                    "LookSide": "Look Side",
+                    "ProjectionId": "Projection ID",
+                    "DeliveryMode": "Delivery Mode",
+                    "AcquisitionStationId": "Acquisition Station ID",
+                    # ProcessingInfo
+                    # "ProcessingLevel":,
+                    # ProductCharacteristics
+                    "AzimuthGeometricResolution": "Azimuth Geometric Resolution",
+                    "GroundRangeGeometricResolution": "Ground Range Geometric Resolution",
+                }
+
+                sbi_field_map = {
+                    "GeoCoordBottomLeft": "Bottom Left Geodetic Coordinates",
+                    "GeoCoordBottomRight": "Bottom Right Geodetic Coordinates",
+                    "GeoCoordTopLeft": "Top Left Geodetic Coordinates",
+                    "GeoCoordTopRight": "Top Right Geodetic Coordinates",
+                    # "GeoCoordTopRightEN": "Top Right East-North",
+                    "NearLookAngle": "Near Look Angle",
+                    "FarLookAngle": "Far Look Angle",
+                }
+
+                def h5_to_str(h5_val):
+                    str_val = str(h5_val)
+                    str_val = str_val.replace("[", "")
+                    str_val = str_val.replace("]", "")
+                    return str_val
+
+                with h5netcdf.File(self._img_path) as netcdf_ds:
+
+                    # Create XML attributes
+                    global_attr = []
+                    for xml_attr, h5_attr in field_map.items():
+                        try:
+                            global_attr.append(
+                                E(xml_attr, h5_to_str(netcdf_ds.attrs[h5_attr]))
+                            )
+                        except KeyError:
+                            # CSG products don't have their ProductName in the h5 file...
+                            if xml_attr == "ProductName":
+                                global_attr.append(
+                                    E(xml_attr, files.get_filename(self._img_path))
+                                )
+
+                    try:
+                        # CSK products
+                        sbi = netcdf_ds.groups["S01"].variables["SBI"]
+                    except KeyError:
+                        # CSG products
+                        sbi = netcdf_ds.groups["S01"].variables["IMG"]
+
+                    for xml_attr, h5_attr in sbi_field_map.items():
+                        global_attr.append(E(xml_attr, h5_to_str(sbi.attrs[h5_attr])))
+
+                    mtd = E.s3_global_attributes(*global_attr)
+                    mtd_el = etree.fromstring(
+                        etree.tostring(
+                            mtd,
+                            pretty_print=True,
+                            xml_declaration=True,
+                            encoding="UTF-8",
+                        )
+                    )
+
+                return mtd_el, {}
+            except KeyError:
+                raise InvalidProductError(
+                    "Missing the XML metadata file. Cannot read the product."
+                )
+
+    def get_quicklook_path(self) -> str:
+        """
+        Get quicklook path if existing.
+
+        Returns:
+            str: Quicklook path
+        """
+        qlk_path = (
+            self._get_band_folder(writable=True) / f"{self.condensed_name}_QLK.tif"
+        )
+        if not qlk_path.is_file():
+            with rasterio.open(str(self._img_path)) as ds:
+                quicklook_paths = [subds for subds in ds.subdatasets if "QLK" in subds]
+
+            if len(quicklook_paths) == 0:
+                LOGGER.warning(f"No quicklook found in {self.condensed_name}")
+            else:
+                rasters.write(
+                    rasters.read(quicklook_paths[0]),
+                    qlk_path,
+                    dtype=np.uint8,
+                    nodata=255,
+                )
+                if len(quicklook_paths) > 1:
+                    LOGGER.info(
+                        "For now, only the quicklook of the first swath is taken into account."
+                    )
+
+        return str(qlk_path)
+
+    @cache
+    def get_orbit_direction(self) -> OrbitDirection:
+        """
+        Get cloud cover as given in the metadata
+
+        .. code-block:: python
+
+            >>> from eoreader.reader import Reader
+            >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
+            >>> prod = Reader().open(path)
+            >>> prod.get_orbit_direction().value
+            "DESCENDING"
+
+        Returns:
+            OrbitDirection: Orbit direction (ASCENDING/DESCENDING)
+        """
+        # Get MTD XML file
+        if isinstance(self.path, CloudPath):
+            h5_xarr_path = BytesIO(self._img_path.read_bytes())
+        else:
+            h5_xarr_path = str(self._img_path)
+
+        with xr.open_dataset(
+            h5_xarr_path, phony_dims="access", engine="h5netcdf"
+        ) as h5_xarr:
+            # Get the orbit direction
+            try:
+                od = OrbitDirection.from_value(getattr(h5_xarr, "Orbit Direction"))
+
+            except TypeError:
+                raise InvalidProductError("Orbit Direction not found in metadata!")
+
+        return od
