@@ -20,29 +20,25 @@ More info `here <https://catalyst.earth/catalyst-system-files/help/references/gd
 """
 import difflib
 import logging
-import warnings
 from datetime import datetime
 from enum import unique
 from typing import Union
 
 import geopandas as gpd
-import rasterio
 from lxml import etree
-from sertit import vectors
+from sertit import files, vectors
 from sertit.misc import ListEnum
 from sertit.vectors import WGS84
 
-from eoreader import cache, cached_property
+from eoreader import cache
 from eoreader.exceptions import InvalidProductError, InvalidTypeError
 from eoreader.products import SarProduct, SarProductType
+from eoreader.products.product import OrbitDirection
 from eoreader.reader import Reader
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME
 
 LOGGER = logging.getLogger(EOREADER_NAME)
 
-# Disable georef warnings here as the SAR products are not georeferenced
-warnings.filterwarnings("ignore", category=rasterio.errors.NotGeoreferencedWarning)
-
 
 @unique
 class Rs2ProductType(ListEnum):
@@ -82,7 +78,7 @@ class Rs2ProductType(ListEnum):
 @unique
 class Rs2SensorMode(ListEnum):
     """
-    Set product default resolution (in meters)
+    Get product default resolution (in meters)
     See here
     <here](https://www.asc-csa.gc.ca/eng/satellites/radarsat/technical-features/radarsat-comparison.asp>`_
     for more information (Beam Modes)
@@ -154,19 +150,6 @@ class Rs2SensorMode(ListEnum):
     """Spotlight Mode"""
 
 
-@unique
-class Rs2Polarization(ListEnum):
-    """
-    RADARSAT-2 polarization mode.
-    Take a look `here <https://catalyst.earth/catalyst-system-files/help/references/gdb_r/RADARSAT-2.html#RADARSAT2__rs2_sfs>`_
-    """
-
-    HH = "HH"
-    VV = "VV"
-    VH = "VH"
-    HV = "HV"
-
-
 class Rs2Product(SarProduct):
     """
     Class for RADARSAT-2 Products
@@ -174,9 +157,9 @@ class Rs2Product(SarProduct):
     You can use directly the .zip file
     """
 
-    def _set_resolution(self) -> float:
+    def _get_resolution(self) -> float:
         """
-        Set product default resolution (in meters)
+        Get product default resolution (in meters)
         """
         # -------------------------------------------------------------
         # Selective Single or Dual Polarization
@@ -244,7 +227,15 @@ class Rs2Product(SarProduct):
 
         return def_res
 
-    def _pre_init(self) -> None:
+    def _set_instrument(self) -> None:
+        """
+        Set instrument
+
+        RADARSAT: https://earth.esa.int/eogateway/missions/radarsat
+        """
+        self.instrument = "SAR C-band"
+
+    def _pre_init(self, **kwargs) -> None:
         """
         Function used to pre_init the products
         (setting needs_extraction and so on)
@@ -252,7 +243,10 @@ class Rs2Product(SarProduct):
         # Private attributes
         self._raw_band_regex = "*imagery_{}.tif"
         self._band_folder = self.path
-        self._snap_path = ""
+        self.snap_filename = ""
+
+        # Its original filename is its name
+        self._use_filename = True
 
         # SNAP can process non-complex archive
         root, nsmap = self.read_mtd()
@@ -267,18 +261,18 @@ class Rs2Product(SarProduct):
         self.needs_extraction = self.product_type == Rs2ProductType.SLC
 
         # Post init done by the super class
-        super()._pre_init()
+        super()._pre_init(**kwargs)
 
-    def _post_init(self) -> None:
+    def _post_init(self, **kwargs) -> None:
         """
         Function used to post_init the products
         (setting product-type, band names and so on)
         """
 
         # Post init done by the super class
-        super()._post_init()
+        super()._post_init(**kwargs)
 
-    @cached_property
+    @cache
     def wgs84_extent(self) -> gpd.GeoDataFrame:
         """
         Get the WGS84 extent of the file before any reprojection.
@@ -289,7 +283,7 @@ class Rs2Product(SarProduct):
             >>> from eoreader.reader import Reader
             >>> path = r"RS2_OK73950_PK661843_DK590667_U25W2_20160228_112418_HH_SGF.zip"
             >>> prod = Reader().open(path)
-            >>> prod.wgs84_extent
+            >>> prod.wgs84_extent()
                                                         geometry
             1  POLYGON ((106.57999 -6.47363, 107.06926 -6.473...
 
@@ -326,7 +320,7 @@ class Rs2Product(SarProduct):
             LOGGER.warning(
                 "Other product types than SGF or SLC haven't been tested for %s data. "
                 "Use it at your own risks !",
-                self.platform.value,
+                self.constellation.value,
             )
 
     def _set_sensor_mode(self) -> None:
@@ -393,7 +387,7 @@ class Rs2Product(SarProduct):
 
         return date
 
-    def _get_name(self) -> str:
+    def _get_name_constellation_specific(self) -> str:
         """
         Set product real name from metadata
 
@@ -404,7 +398,7 @@ class Rs2Product(SarProduct):
 
         # Test filename
         reader = Reader()
-        if not reader.valid_name(name, self._get_platform()):
+        if not reader.valid_name(name, self._get_constellation()):
             LOGGER.warning(
                 "This RADARSAT-2 filename is not valid. "
                 "However RADARSAT-2 files do not provide anywhere the true name of the product. "
@@ -434,3 +428,54 @@ class Rs2Product(SarProduct):
         mtd_archived = r"product\.xml"
 
         return self._read_mtd_xml(mtd_from_path, mtd_archived)
+
+    def get_quicklook_path(self) -> str:
+        """
+        Get quicklook path if existing.
+
+        Returns:
+            str: Quicklook path
+        """
+        quicklook_path = None
+        try:
+            if self.is_archived:
+                quicklook_path = files.get_archived_rio_path(
+                    self.path, file_regex=r".*BrowseImage\.tif"
+                )
+            else:
+                quicklook_path = str(next(self.path.glob("BrowseImage.tif")))
+        except (StopIteration, FileNotFoundError):
+            LOGGER.warning(f"No quicklook found in {self.condensed_name}")
+
+        return quicklook_path
+
+    @cache
+    def get_orbit_direction(self) -> OrbitDirection:
+        """
+        Get cloud cover as given in the metadata
+
+        .. code-block:: python
+
+            >>> from eoreader.reader import Reader
+            >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
+            >>> prod = Reader().open(path)
+            >>> prod.get_orbit_direction().value
+            "DESCENDING"
+
+        Returns:
+            OrbitDirection: Orbit direction (ASCENDING/DESCENDING)
+        """
+        # Get MTD XML file
+        root, nsmap = self.read_mtd()
+        namespace = nsmap[None]
+
+        # Get the orbit direction
+        try:
+            od = OrbitDirection.from_value(
+                root.findtext(f".//{namespace}passDirection").upper()
+            )
+
+        except TypeError:
+            raise InvalidProductError("passDirection not found in metadata!")
+
+        return od
