@@ -919,7 +919,7 @@ class SarProduct(Product):
         resolution: Union[float, tuple] = None,
         size: Union[list, tuple] = None,
         resampling: Resampling = Resampling.bilinear,
-    ) -> str:
+    ) -> Union[Path, CloudPath]:
         """
         Compute Hillshade mask
 
@@ -929,13 +929,13 @@ class SarProduct(Product):
             resampling (Resampling): Resampling method
             size (Union[tuple, list]): Size of the array (width, height). Not used if resolution is provided.
         Returns:
-            str: Hillshade mask path
+            Union[Path, CloudPath]: Hillshade mask path
         """
         raise InvalidProductError("Impossible to compute hillshade mask for SAR data.")
 
     def _has_cloud_band(self, band: BandNames) -> bool:
         """
-        Does this products has the specified cloud band ?
+        Does this product has the specified cloud band ?
 
         .. code-block:: python
 
